# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

########################################
# DESCRIPTION: Process the output from calculations into nice tables and graphs.
# WORKING DIR:
# INPUT (MEMORY):
#   -
# INPUT (FILES):
#   -
# OUTPUT (MEMORY):
#   -
# OUTPUT (FILES):
#   -
########################################
library(dplyr)
library(tidyr)
library(ggplot2)
options(dplyr.summarise.inform = FALSE)

fn <- new.env()
source('src/fn_CR.R', local = fn)

#Single run of LCMCR with interesting "non-identifiable" distribution
samps <- readRDS('output/single_run_interesting.rds')
#remove the case of N=10^6 because of poor mixing
(p <-
  as.data.frame(samps$samples) %>%
  gather(., key = 'N', value = 'p0') %>%
  separate(N, c('X', 'N'), sep = '_')%>%
  mutate(N = gsub('\\.','',N) %>% as.integer %>% format(big.mark = ',')) %>%
    ggplot() +
    theme_gray() +
    ggtitle('Posterior density of the probability of no-capture', subtitle = 'using a non-identifiable model') +
    aes(x = N, y = p0) +
    xlab('Population Size (N)') +
    ylab('p0') +
    ylim(0,1) +
    geom_violin() +
    geom_hline(aes(yintercept = samps$params$p0A), lty = 2, color = 'gray30', show.legend = TRUE) +
    geom_hline(aes(yintercept = samps$params$p0B), lty = 3, color = 'gray30', show.legend = TRUE) +
    #annotate(geom = 'text', x = 0.7, y = samps$params$p0A + 0.02, label = paste0('p0(1) = ', format(samps$params$p0A, digits= 3))) +
    #annotate(geom = 'text', x = 0.7, y = samps$params$p0B + 0.02, label = paste0('p0(2) = ', format(samps$params$p0B, digits= 3))) +
    scale_linetype_manual(breaks = c('model 1', 'model 2'), values = 2:3) +
    theme(legend.position = c("top", 'left'))
)
ggsave(filename = 'output/densities_interesting.pdf', p, width = 15, height = 10, units = 'cm')
ggsave(filename = 'output/densities_interesting.png', p, width = 15, height = 10, units = 'cm')

#saveRDS(p, file = 'output/proc_samps_interesting.R')
#write('output/proc_samps_interesting.R', file = 'output/process_output.files')
##############################
# Recalculation of examples with appropriately long runs
experim <- list(
  'output/repeated_3a.rds',
  'output/repeated_3b.rds',
  'output/repeated_4a.rds',
  'output/repeated_4b.rds',
  'output/repeated_4c.rds',
  'output/repeated_interesting.rds'
)
aa <- lapply(experim, FUN = function(z) { readRDS(z) })
names(aa) <- lapply(aa, function(x) strsplit(x$meta$trial_par$filename, '[/\\.]') %>%
         lapply(., function(x)x[[2]]) ) %>% unlist

dat_exper <- lapply(names(aa), FUN = function(z) {
  lapply(names(aa[[z]]$summaries), FUN = function(x) {
    lapply(names(aa[[z]]$summaries[[x]]), FUN = function(y) {
      c(n = aa[[z]]$summaries[[x]][[y]]$mean_n, mean = aa[[z]]$summaries[[x]][[y]]$mean_p0, aa[[z]]$summaries[[x]][[y]]$quant) %>%
        as.list %>% as.data.frame
    }) %>% do.call('rbind', .) %>%
      mutate(N = x)
  }) %>%  do.call('rbind', .) %>%
    mutate(exper = z,
           p0 = fn$probs_zero(aa[[z]]$meta$lcm_par$pi, aa[[z]]$meta$lcm_par$lambda)
    )
})%>% do.call('rbind', .)

experim_meta <- sapply(aa, FUN = function(x){
  x$meta
}, simplify = FALSE, USE.NAMES = TRUE)
saveRDS(experim_meta, file = 'output/proc_experiment_metadata.rds')

results_experim <- dat_exper %>%
  mutate(cover = p0 > X2.5. & p0 < X97.5.) %>%
  mutate(length =  X97.5. - X2.5.) %>%
  separate(N, c('X', 'N'), sep = '_')%>%
  mutate(N = gsub('\\.','',N) %>% as.integer) %>%
  group_by(exper, N) %>%
  summarize(n = mean(n), p0 = max(p0), mean = mean(mean),
            cover = mean(cover),
            length = mean(length)
            ) %>%
  ungroup %>%
  print
saveRDS(results_experim, file = 'output/proc_table_res_repeated_exp.rds')

probs <- sapply(aa, function(x) {
  x$meta$lcm_par %>%
    { data.frame(
      pi = .$pi,
      p0_z = { fn$probs_zero_by_class(.$pi, .$lambda) },
      p0 = { fn$probs_zero_by_class(.$pi, .$lambda) * .$pi }
      ) } %>%
    mutate(p1_z = 1- p0_z, p1 = pi * p1_z)
  }, simplify = FALSE, USE.NAMES = TRUE) %>%
  print
saveRDS(probs, file = 'output/proc_table_res_probs_exp.rds')
