# 
# Copyright (C) 2022 Daniel Manrique-Vallier
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

########################################
# DESCRIPTION: *Single run* simulation with parameters from output/params_intersting.rds
# WORKING DIR: ../
# INPUT (FILES):
#   - src/fn_CR.R
# OUTPUT (FILES):
#   - output/samps_intersting.rds
########################################
#debugging <- TRUE; quick <- TRUE
quick <- Sys.getenv("MK_QUICK_RUN") == "TRUE"

library(LCMCR)
source('src/fn_CR.R')
if (exists("debugging")){
  #debugging version of parameters
  src <- 'batches/params_interesting_J3K2.rds'
  dst <- 'output/single_run_interesting_J3K2.rds'
} else {
  x <- commandArgs(trailingOnly = TRUE)
  src <- x[1]
  dst <- x[2]
  if (src == "" || dst == "") stop("Need to specify source and destination file names.")
}
par <- readRDS(src)
pi1 <- par$par1$pi
lambda1 <- par$par1$lambda
if (quick){
  Ns <- c(2e3, 1e4, 1e5)
  samples <- 10
  th <- 1
  burnin <- 1
} else {
  Ns <- c(2e3, 1e4, 1e5)
  samples <- 10000
  th <- 10000
  burnin <- 100000
}

library(snowfall)
sfInit(parallel = TRUE, cpus = 20)
sfLibrary(LCMCR)
sfSource('src/fn_CR.R')
sfExport('par','pi1','lambda1','samples','th','burnin')

r <- sfLapply(Ns, fun = function(N) {
  set.seed(N)
  a <- generate_lcmpop(pi1, lambda1, N)
  x <- lcmCR(captures = a, in_list_label='Yes', not_in_list_label='No',buffer_size = samples, thinning = th, seed = 2, K = 10, verbose = F)
  x$Set_Trace('prob_zero')
  lcmCR_PostSampl(x, burnin = burnin, thinning = th,
                  clear_buffer = TRUE, output = FALSE)
  n <- NROW(a)
  chain <- x$Get_Trace('prob_zero')
  return( list( N = N, n = n, chain = chain ))
  }
)

res <- list(
            params = par,
            N = lapply(r, FUN = function(x){ return(x$N) }),
            n = lapply(r, FUN = function(x){ return(x$n) }),
            samples = lapply(r, FUN = function(x){ return(x$chain) }),
            niter = samples * th,
            thinning = th,
            burnin = 10000,
            src = src,
            date = date()
)
names(res$samples) <- paste0('N_',res$N)
saveRDS(res, file = dst)

